// TaskChannel.cpp: implementation of the CTaskChannel class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TaskPlot.h"
#include "color.h"
#include "TaskChannel.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

int issame(double x,double z);
int bequal(double x,double z);
int lequal(double x,double z);


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

/////////////////////////////////////////
CTaskChannel::CTaskChannel()
{ 
	m_nColor		= DKGRAY;	// regend color	
	m_nLineStyle	= PS_SOLID;	
	m_nLineWidth	= 0;	
	m_sName			= "";		
	m_sDescription	= "";	
	m_pValues		= NULL;		
	m_nSize			= 0; 
	m_cur_index		= 0;
}

bool CTaskChannel::Reset(DWORD size)
{
	assert(size > 0);
	if (m_pValues) delete(m_pValues);
	m_pValues  = (CTaskEventValue*)new CTaskEventValue[ size ];
	if(!m_pValues) 
		return false;

	m_nSize = size;
	m_inc_unit = size;
	return true;
}

void CTaskChannel::AddValue(double time, int event)
{
	if (m_pValues == NULL) return;

	if (m_cur_index >= m_nSize)
	{
		int maxsize = m_nSize + m_inc_unit;

		CTaskEventValue *pNewArray =  new CTaskEventValue[ maxsize ];
		for( int i =0; i < (long)m_nSize; i++ )
			pNewArray[ i ] = m_pValues[ i ];
		delete []m_pValues;
		m_pValues = pNewArray;
		m_nSize = maxsize;
	}

	m_pValues[m_cur_index].event = (EnumTaskEvent) event;
	m_pValues[m_cur_index].time	= time;

	m_cur_index ++;
}

CTaskChannel& CTaskChannel::operator =(CTaskChannel& from)
{
	int i;
	if( this == &from ) return *this;

	if(m_pValues)		delete []m_pValues;
	m_nSize = from.GetSize();
	
	m_pValues = new CTaskEventValue[m_nSize];
	for(i=0; i < (int)m_nSize; i++)
		m_pValues[i] = from.m_pValues[i];

	m_nColor		= from.m_nColor;		
	m_nLineStyle	= from.m_nLineStyle;	
	m_nLineWidth	= from.m_nLineWidth;	
	m_sName			= from.m_sName;		
	m_sDescription	= from.m_sDescription;	

	m_period		= from.m_period;
	m_deadline		= from.m_deadline;
	m_priority		= from.m_priority;
	m_wcet			= from.m_wcet;
	m_offset		= from.m_offset;

	m_cur_index		= 0;

	return *this;
}

void CTaskChannel::SortTimeLine()
{
	int nPoint = GetSize();
	EnumTaskEvent tempe;
	double tempt;

	for(int i=0; i< nPoint-1; i++)
	{
		for(int j=i+1; j< nPoint; j++)
		{
			if(ReverseOrder(m_pValues[i], m_pValues[j]))
			{
				tempe = (EnumTaskEvent)m_pValues[i].event;
				tempt = m_pValues[i].time;

				m_pValues[i].event = m_pValues[j].event;
				m_pValues[i].time = m_pValues[j].time;

				m_pValues[j].event = tempe;
				m_pValues[j].time = tempt;
			}
		}
	}
/*
	for(i=0; i< nPoint; i++)
	{
		CString temp;
		temp.Format("time[%d]=%.2lf, event=%d \r\n",i,m_pValues[i].time, m_pValues[i].event);
		OutputDebugString((LPCSTR)temp);
	}
	OutputDebugString("--------");
	*/
}

/* 
If B should be prior to A, then return 1
*/
int CTaskChannel::ReverseOrder(CTaskEventValue &a, CTaskEventValue &b)
{
	if(b.time < a.time)
	{ // reverse this.
		return 1;
	}
	else
	{
		if(b.time == a.time)
		{
			if(b.event == COMPLETE_EVENT)
			{
				if(a.event == RELAESE_EVENT)
				{
					return 1;
				}
				else if(a.event == START_EVENT)
				{
					return 1;
				}
			}
			else if(b.event == RELAESE_EVENT)
			{
				if(a.event == START_EVENT)
				{
					return 1;
				}
			}
			else if(b.event == RESUME_EVENT)
			{
				if(a.event == PREEMPTED_EVENT)
				{
					return 1;
				}
			}
			else if(b.event == DEADLINE_EVENT)
			{
				if(a.event != DEADLINE_EVENT)
				{
					return 1;
				}
			}
		}
	}
	return 0;
}

void CTaskChannel::PrintTimeLine(CString &out)
{
	int nPoint = GetSize();
	EnumTaskEvent event;
	double time;

	CString temp;

	temp.Format("task -------- \r\n");
	out += temp;

	temp.Format("P:%.3lf, D:%.3lf, C:%.3lf, S:%.3lf \r\n",m_period,m_deadline,m_wcet,m_offset);
	out += temp;

	for(int i=0; i< nPoint-1; i++)
	{
		event = m_pValues[i].event;
		time = m_pValues[i].time;

		temp.Format("%.2f, %d\r\n", time, event);
		out += temp;		
	}
}

void CTaskChannel::SetPeriodicEvents()
{
	double t;
	if(m_period < 1.0E-5) return;

	for(t = 0; lequal(t,m_max_time);)
	{
		AddValue(t, RELAESE_EVENT);
		t += m_period;
	}

	for(t = m_offset;lequal(t,m_max_time);)
	{
		AddValue(t, START_EVENT);
		t += m_period;
	}

	if(m_deadline < 1.0E-5)
	{
		m_deadline = m_period;
	}

	for(t = 0; lequal(t,m_max_time);)
	{
		AddValue(t + m_deadline, DEADLINE_EVENT);
		t += m_period;
	}
}

int CTaskChannel::ExistInPreemptedTime(double time)
{
	double time1, time2;

	if(ExistEvent(time, PREEMPTED_EVENT))// raising edge
		return 1;

	if(ExistEvent(time, RESUME_EVENT))// falling edge --> end of preemption
		return 0; 

	time1 = FindNearEventTime(time, PREEMPTED_EVENT, -1);
	time2 = FindNearEventTime(time, RESUME_EVENT, -1);

	if(time1 > time2)
		return 1;
	return 0;
}

/*
*/
int CTaskChannel::GetFreeTimeBound(double start, double duration, TimeBound *ret, int sizeofret)
{
	double time2,laptime;

	laptime = 0;
	int iLoop = 0;
	while(1)
	{
		start = FindNextAvailableTime(start);
		time2 = FindNearEventTime(start, PREEMPTED_EVENT, 1);
		if(time2 > 0)
		{
			if(iLoop >= sizeofret)
			{
				MessageBox(NULL,"Error","GetFreeTimeBound() needs more memory",MB_OK);
				return 0;
			}

			laptime = time2 - start;
			if(laptime < duration)
			{
				ret[iLoop].x = start;
				ret[iLoop].y = time2;
	
				duration = duration - laptime;

				start = FindNearEventTime(time2, RESUME_EVENT, 1);
			}
			else
			{
				ret[iLoop].x = start;
				ret[iLoop].y = start + duration;
				
				return iLoop+1;
			}
		}
		else
		{
			ret[iLoop].x = start;
			ret[iLoop].y = start + duration;
			
			return iLoop+1;
		}

		iLoop++;
	}
}

void CTaskChannel::SetPreemptionEvents(CTaskChannel *highTask)
{
//	if(highTask->m_priority < m_priority)
//		return;
	double ht,lt;
	TimeBound bounds[50],B1,B2,B3,B4;
	int ret,i,nInt;
	int sizeofbounds,bWritten;

	sizeofbounds  = sizeof(bounds)/sizeof(bounds[0]);

	for(ht = highTask->m_offset; lequal(ht,m_max_time);)
	{
		ret = highTask->GetFreeTimeBound(ht,highTask->m_wcet,bounds, sizeofbounds);

		for(i=0; i<ret; i++)
		{
			// check the time bound is inside of working time.
			// [Release ~ Start] is not working time

			bWritten = 0;
			B2 = bounds[i];
			for(lt = 0; lt <= m_max_time;)
			{
				B1.x = lt;
				B1.y = lt + m_offset;

				nInt = PreemptRegion(B3,B2,B1,B4);

				if(nInt == -1)
					bWritten = 1;

				if(nInt > 0)
				{
					AddValue(B3.x, PREEMPTED_EVENT);
					AddValue(B3.y, RESUME_EVENT);
					bWritten = 1;
				}
				if(nInt > 1)
				{
					AddValue(B4.x, PREEMPTED_EVENT);
					AddValue(B4.y, RESUME_EVENT);
					bWritten = 1;
				}

				lt += m_period;
			}

			if(bWritten == 0)
			{
				AddValue(B2.x, PREEMPTED_EVENT);
				AddValue(B2.y, RESUME_EVENT);
			}
		}

		SortTimeLine();	

		ht += highTask->m_period;
	}
}

void CTaskChannel::SetCompleteEvent()
{
	double t;
	static TimeBound bounds[64];
	int ret;
	int sizeofbounds;

	sizeofbounds  = sizeof(bounds)/sizeof(bounds[0]);
	for(t = m_offset; t <= m_max_time;)
	{
		ret = GetFreeTimeBound(t, m_wcet, bounds, sizeofbounds);
		if(ret > 0)
			AddValue(bounds[ret-1].y , COMPLETE_EVENT);

		t += m_period;
	}
}

double CTaskChannel::FindNextAvailableTime(double start)
{
	int bPreempted;
	double start2;

	bPreempted = ExistInPreemptedTime(start);
	while(bPreempted)
	{
		start2 = FindNearEventTime(start, RESUME_EVENT, 1);
		if(start2 < start)
		{
			MessageBox(NULL,"wrong","Error", MB_OK);
		}
		start = start2;
		bPreempted = ExistInPreemptedTime(start);
	}
	
	return start;
}

/* find a event at a given time*/
int CTaskChannel::ExistEvent(double aTime, int aEvent)
{
	int event,nPoint,i;
	double time;

	nPoint = GetSize();
	for(i=0; i< nPoint; i++)
	{
		event = m_pValues[i].event;
		time = m_pValues[i].time;
		if(aTime == time)
		{
			if(aEvent == event)
				return 1;
		}
	}

	return 0;
}

/*
dir > 0: forward search of aEvent
dir < 0: backward search of aEvent
*/
double CTaskChannel::FindNearEventTime(double aTime, int aEvent, int dir)
{
	int event,nPoint,i,nearIndex,j;
	double time;
	double tem;

	nPoint = GetSize();
	for(i=0; i< nPoint; i++)
	{
		time = m_pValues[i].time;
		if(dir < 0)
		{// backward search
			tem = aTime - time;
			if(lequal(tem,0))
			{
				nearIndex = 0;
				if(i > 0)
					nearIndex = i - 1;

				for(j=nearIndex; j > -1; j--)
				{
					event = m_pValues[j].event;
					time = m_pValues[j].time;
					if(aEvent == event)
					{
						return time;
					}
				}	
			}
		}
		else
		{ // forward search
			tem = aTime - time;
			if(tem < 0)
			{
				nearIndex = i;

				for(j=nearIndex; j < nPoint; j++)
				{
					event = m_pValues[j].event;
					time = m_pValues[j].time;
					if(aEvent == event)
					{
						return time;
					}
				}	
			}
		}
	}

	return 0;
}

/*
B3 = B2 - B1
B4 is used when B3 is split.
*/
int CTaskChannel::PreemptRegion(TimeBound &B3, TimeBound &B2, TimeBound &B1, TimeBound &B4)
{
	// x is min, y is max
	B3 = B2;

	// min-max test; no overlapp case
	if(B2.x > B1.y) return 0;
	if(B1.x > B2.y) return 0;

	//case 1. B1 is same or larger
	if(bequal(B2.x,B1.x))
	if(bequal(B1.y, B2.y))
	{
		return -1;
	}

	// case 2. B1 is on left side intersect.
	if(bequal(B2.x, B1.x))
	if(bequal(B1.y, B2.x))
	{
		B3.x = B1.y; // front part of B2 is cut.
		B3.y = B2.y;
		
		return 1;
	}

	// case 3. B1 is on right side intersect.
	if(bequal(B2.y, B1.x))
	if(bequal(B1.y, B2.y))
	{
		B3.x = B2.x;
		B3.y = B1.x; // rear part of B2 is cut.
		
		return 1;
	}

	// case 4. B1 is inside of B2
	if(bequal(B1.x, B2.x))
	if(bequal(B2.y, B1.y))
	{
		B3.x = B2.x;
		B3.y = B1.x; // rear part of B2 is cut.

		B4.x = B1.y;
		B4.y = B2.y;

		return 2;
	}

	MessageBox(NULL,"Something Wrong","PreemptRegion()", MB_OK);
	return 0;
}

