// TaskChannel.h: interface for the CTaskChannel class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TASKCHANNEL_H__4ACFB21D_62BF_4065_AA23_49EE1DCD9FA1__INCLUDED_)
#define AFX_TASKCHANNEL_H__4ACFB21D_62BF_4065_AA23_49EE1DCD9FA1__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#include "Str.h"
#include "ClassArr.h"

enum   EnumTaskEvent	{	RELAESE_EVENT=1,
							START_EVENT,
							COMPLETE_EVENT,
							PREEMPTED_EVENT,
							BLOCK_EVENT,
							RESUME_EVENT,
							LOCK_EVENT,
							UNLOCK_EVENT,
							DEADLINE_EVENT,
							MAX_EVENT_NUMBER,
};

typedef struct 
{
	double x,y;
} TimeBound;

struct CTaskChannel  
{
	struct CTaskEventValue
	{
		EnumTaskEvent	event;
		double			time;
	};

	COLORREF	m_nColor;			// line color
	int			m_nLineStyle;		// line style
	int			m_nLineWidth;		// line width
	CStr		m_sName;			// name
	CStr		m_sDescription;		// description
	CStr		m_sUnit;			// unit
	double		m_dMin;				// minimum value
	double		m_dMax;				// maximum value
	double		m_dScaleLow;		// scaled minimum value
	double		m_dScaleHigh;		// scaled maximum value

	// RT task properties
	double		m_wcet;
	double		m_period;
	double		m_deadline;
	double		m_jitter;
	double		m_preemption;
	double		m_offset;
	int			m_priority;
	CTaskEventValue*	m_pValues;
	////////////////////////////////////
	double		m_max_time;

	int PreemptRegion(TimeBound &B3, TimeBound &B2, TimeBound &B1, TimeBound &B4);
	void SetPeriodicEvents();
	void SetPreemptionEvents(CTaskChannel *highTask);
	void SortTimeLine();
	void PrintTimeLine(CString &out);
	int ExistEvent(double aTime, int aEvent);
	int ExistInPreemptedTime(double time);
	int GetFreeTimeBound(double start, double duration, TimeBound *ret, int sizeofret);
	double FindNearEventTime(double aTime, int aEvent, int dir);
	void SetCompleteEvent();
	double FindNextAvailableTime(double start);
	int ReverseOrder(CTaskEventValue &a, CTaskEventValue &b);
	void AddValue(double time, int event);

	CTaskChannel();
	~CTaskChannel()	{ if (m_pValues)  delete (m_pValues); }

	// Operation
	int GetSize() { return m_cur_index;} 
	void SetLineColor(COLORREF cr)		{ m_nColor = cr; }
	COLORREF GetLineColor()				{ return m_nColor; }
	void SetLineStyle(int ls)			{ m_nLineStyle = ls; }
	int  GetLineStyle()					{ return m_nLineStyle; }
	void SetLineWidth(int wd)			{ m_nLineWidth = wd; }
	int  GetLineWidth()					{ return m_nLineWidth; }
	void SetLineName(char* name)		{ m_sName = name; }
	const char* GetLineName()			{ return m_sName.GetChar(); }
	void SetLineDescription(char* desc)	{ m_sDescription = desc; }
	const char* GetLineDescription()	{ return m_sDescription.GetChar(); }
	void SetLineUnit(char* unit)		{ m_sUnit = unit; }
	const char* GetLineUnit()			{ return m_sUnit.GetChar(); }
	void SetLineMinValue(double min)	{ m_dMin = min; }
	double GetLineMinValue()			{ return m_dMin; }
	void SetLineMaxValue(double max)	{ m_dMax = max; }
	double GetLineMaxValue()			{ return m_dMax; }
	void SetLineScaleLow(double low)	{ m_dScaleLow = low; }
	double GetLineScaleLow()			{ return m_dScaleLow; }
	void SetLineScaleHigh(double high)	{ m_dScaleHigh = high; }
	double GetLineScaleHigh()			{ return m_dScaleHigh; }
	bool Reset(DWORD size);
	CTaskChannel& operator =(CTaskChannel& from);

private:
	DWORD		m_nSize;		// number of points in the line
	DWORD		m_cur_index;
	DWORD		m_inc_unit;
};

#endif // !defined(AFX_TASKCHANNEL_H__4ACFB21D_62BF_4065_AA23_49EE1DCD9FA1__INCLUDED_)
